#! /usr/bin/env python

import logtool
from xxpaper.sheet import Sheet

class Train (Sheet):

  @logtool.log_call
  def __init__ (self, cfgs, paper, outline, sheet, page, fname):
    super (Train, self).__init__ (cfgs, paper, outline, sheet, page, fname)
    # Offsets within tile
    self.rust_stripe_inset_y = int (self.value ("rust_stripe_inset_y"))
    self.rust_stripe_height = int (self.value ("rust_stripe_height"))

  @logtool.log_call
  def page_details (self):
    self.rust_stripe ()
    self.trade_stripe ()

  @logtool.log_call
  def tile_details (self, x, y):
    self.train_type (x, y)
    self.rust (x, y)
    self.trade_to (x, y)
    self.train_cost (x, y)

  @logtool.log_call
  def train_cost (self, x, y):
    bx = self.tile_x / 2
    by = self.rust_stripe_inset_y + (self.rust_stripe_height * 4)
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("cost", x, y, h_centre = 0, v_centre = 1)

  @logtool.log_call
  def trade_to (self, x, y):
    bx = self.tile_x / 2
    by = (self.rust_stripe_inset_y * 2) + self.rust_stripe_height + 3
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("trade", x, y, h_centre = 0, v_centre = 1)

  @logtool.log_call
  def trade_stripe (self):
    oy = self.y_off + (self.rust_stripe_inset_y * 2) + self.rust_stripe_height
    for y in xrange (self.num_y):
      by = (y * self.tile_y) + oy
      self.box ("trade_stripe", 0, y, 0, by,
                self.rubber_x, self.rust_stripe_height)

  @logtool.log_call
  def rust (self, x, y):
    bx = self.tile_x / 2
    by = self.rust_stripe_inset_y + 2
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("rust", x, y, h_centre = 0, v_centre = 1)

  @logtool.log_call
  def rust_stripe (self):
    oy = self.y_off + self.rust_stripe_inset_y
    for y in xrange (self.num_y):
      by = (y * self.tile_y) + oy
      self.box ("rust_stripe", 0, y, 0, by,
                self.rubber_x, self.rust_stripe_height)

  @logtool.log_call
  def train_type (self, x, y):
    inset_y = int (self.value ("train_type_inset_y"))
    bx = self.tile_x / 2
    by = self.tile_y - inset_y
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("train", x, y, h_centre = 0, v_centre = 1)



#! /usr/bin/env python

from __future__ import absolute_import
import clip, fnmatch, itertools, jinja2, logging, logtool
import os, pkg_resources, sys
from addict import Dict
from configobj import ConfigObj
from functools import partial
from path import Path
from StringIO import StringIO
from .cmdio import CmdIO
import xxpaper

logging.basicConfig (level = logging.WARN)
LOG = logging.getLogger (__name__)
APP = clip.App (name = "xxpaper")
CONFIG = Dict ({
  "directory": Path ("./"),
  "formats": ["outline", "nooutline",],
  "game_fname": None,
  "nocolour": False,
  "pages": ["*",],
  "papers": ["*",],
  "sections": ["*",],
  "template": None,
  "quiet": False,
  "verbose": False,
})
IO = CmdIO (CONFIG)

@logtool.log_call (log_args = False)
def make (cfgs, paper, outline, section, page, fname):
  with getattr (xxpaper, section.capitalize ()) (
      cfgs, paper, outline, section, page, fname) as t:
    t.make ()
  IO.info (fname)

@logtool.log_call
def match_filter (name, filters):
  for f in filters:
    if fnmatch.fnmatch (name, f):
      return True
  return False

@logtool.log_call
def get_cfgval (cfgs, section, name):
  for cfg in itertools.chain (cfgs):
    try:
      return cfg[section][name]
    except: # pylint: disable=bare-except
      pass
    try:
      return cfg["DEFAULT"][name]
    except: # pylint: disable=bare-except
      pass
  clip.exit ("Error: Cannot find the value of: %s" % name, err = True)

@logtool.log_call (log_args = False, log_rc = False)
def load_configs ():
  fname = CONFIG.game_fname
  raw = fname.bytes ().decode ('unicode_escape')
  env = jinja2.Environment(extensions = ["jinja2.ext.loopcontrols",
                                         "jinja2.ext.with_",])
  xxp = StringIO (env.from_string (raw).render ())
  if CONFIG.template:
    fn = CONFIG.directory / fname.namebase + "_expanded.cfg"
    IO.debug ("Expanded template: %s" % fn)
    with open (fn, "w") as f:
      f.write (xxp.getvalue ())
  game = ConfigObj (xxp.readlines ())
  # conf = ConfigObj (file (fname).readlines ())
  if "DEFAULT" not in game.sections:
    game["DEFAULT"] = {}
  game["DEFAULT"]["source_filename"] = "File: %s" % CONFIG.game_fname
  runtime = ConfigObj (["[DEFAULT]",])
  defdata = pkg_resources.resource_string ("xxpaper", "DEFAULT.conf")
  defproc = StringIO (env.from_string (defdata).render ())
  default = ConfigObj (defproc.readlines ())
  return runtime, game, default

@logtool.log_call
def option_dir (directory):
  CONFIG.directory = Path (directory)
  CONFIG.directory.makedirs_p ()

@logtool.log_call
def option_logging (flag): # pylint: disable=unused-argument
  logging.root.setLevel (logging.DEBUG)

@logtool.log_call
def option_list (option, value):
  CONFIG[option] = [s.strip () for s in value.split (",")]

@logtool.log_call
def option_setopt (option, value):
  CONFIG[option] = value

@logtool.log_call
def option_version (opt): # pylint: disable = W0613
  clip.echo (xxpaper.__version__)
  sys.exit (0)

@logtool.log_call
def arg_game_file (fname):
  fname = Path (fname)
  if not fname.isfile ():
    clip.exit ("Game file not found: %s" % fname, err = True)
  CONFIG.game_fname = fname

@APP.main (name = Path (sys.argv[0]).basename (),
           description = "18xx rapid prototyping tool",
           tree_view = "-H")
@clip.flag ('-H', '--HELP',  help = "Help for all sub-commands")
@clip.flag ("-C", "--nocolour", name = "nocolour",
            help = "Suppress colours in reports",
            callback = partial (option_setopt, "nocolour"))
@clip.flag ("-D", "--debug", name = "debug", help = "Enable debug logging",
            callback = option_logging)
@clip.opt ("-d", "--dir", name = "directory", help = "Directory to process",
           callback = option_dir)
@clip.opt ("-f", "--formats", name = "formats", help = "Art formats",
           callback = partial (option_list, "formats"))
@clip.opt ("-P", "--papers", name = "papers", help = "Paper sizes",
           required = False, callback = partial (option_list, "papers"))
@clip.opt ("-p", "--pages", name = "pages", help = "Pages to process",
           required = False, callback = partial (option_list, "pages"))
@clip.flag ("-q", "--quiet", name = "quiet",
            help = "Suppress information messages",
            callback = partial (option_setopt, "quiet"))
@clip.opt ("-s", "--sections", name = "sections", help = "Sections to process",
           required = False, callback = partial (option_list, "sections"))
@clip.flag ("-t", "--template", name = "template", help = "Save template file",
           required = False, callback = partial (option_setopt, "template"))
@clip.flag ("-v", "--verbose", name = "verbose",
            help = "Extra information messages",
            callback = partial (option_setopt, "verbose"))
@clip.flag ("-V", "--Version", help = "Report installed version",
            callback = option_version)
@clip.arg (name = "game_file", nargs = 1, default = None,
           help = "XXPaper game file", required = True,
           callback = arg_game_file)
@logtool.log_call
def app_main (*args, **kwargs): # pylint: disable=unused-argument
  if not CONFIG.conf.debug:
    logging.basicConfig (level = logging.ERROR)
  if not sys.stdout.isatty ():
    option_setopt ("nocolour", True)
  runtime, game, default = load_configs ()
  cfgs = [runtime, game, default,]
  formats = (CONFIG.formats if CONFIG.formats
              else get_cfgval (cfgs, "DEFAULT", "formats"))
  for paper in get_cfgval (cfgs, "DEFAULT", "papers"):
    if not match_filter (paper, CONFIG.papers):
      continue
    IO.debug ("Paper size: %s" % paper)
    for form in formats:
      runtime["DEFAULT"]["outline"] = form
      IO.debug ("  Format: %s" % form)
      for section in game.sections:
        if section == "DEFAULT" or not match_filter (section, CONFIG.sections):
          continue
        IO.debug ("    Section: %s" % section)
        for page in game[section].sections:
          IO.debug ("      Page: %s" % page)
          if not match_filter (page, CONFIG.pages):
            continue
          make (cfgs, paper, form, section, page,
                os.path.join ("./", "%s_%s-%s-%s.ps"
                              % (section, page, form, paper)))

@logtool.log_call
def main ():
  try:
    APP.run ()
  except KeyboardInterrupt:
    pass
  except clip.ClipExit:
    sys.exit (0)
  except Exception as e:
    logtool.log_fault (e)
    IO.error ("Something broke!")
    sys.exit (1)

if __name__ == "__main__":
  main ()



#! /usr/bin/env python

import logtool
from xxpaper.sheet import Sheet

class Private (Sheet):

  @logtool.log_call (log_args = False)
  def __init__ (self, cfgs, paper, outline, sheet, page, fname):
    super (Private, self).__init__ (cfgs, paper, outline, sheet, page, fname)
    # Offsets within tile
    self.number_stripe_height = float (self.value ("number_stripe_height"))
    self.number_text_inset_x = float (self.value ("number_text_inset_x"))
    self.number_stripe_inset_y = float (self.value ("number_stripe_inset_y"))

  @logtool.log_call
  def page_details (self):
    self.number_stripe ()

  @logtool.log_call
  def tile_details (self, x, y):
    self.title (x, y)
    self.description (x, y)
    self.number_cost (x, y)
    self.number_revenue (x, y)
    self.closes (x, y)

  @logtool.log_call
  def number_stripe (self):
    oy = self.y_off + self.number_stripe_inset_y
    for y in xrange (self.num_y):
      by = (y * self.tile_y) + oy
      self.box ("number", 0, 0, 0, by, self.rubber_x,
                self.number_stripe_height)

  @logtool.log_call
  def number_cost (self, x, y):
    bx = self.number_text_inset_x
    by = self.number_stripe_inset_y + 3
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("cost", x, y, h_centre = -1, v_centre = 1)

  @logtool.log_call
  def number_revenue (self, x, y):
    bx = self.tile_x - self.number_text_inset_x
    by = self.number_stripe_inset_y + 3
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("revenue", x, y, h_centre = 1, v_centre = 1)

  @logtool.log_call
  def title (self, x, y):
    inset_y = float (self.value ("title_inset_y"))
    bx = self.tile_x / 2
    by = self.tile_y - inset_y
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("title", x, y, h_centre = 0, v_centre = 1)

  @logtool.log_call
  def description (self, x, y):
    bx = self.tile_x / 2
    by = (self.number_stripe_inset_y * 2) + self.number_stripe_height
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("desc", x, y, h_centre = 0, v_centre = -1)

  @logtool.log_call
  def closes (self, x, y):
    inset_x = float (self.value ("close_inset_x"))
    inset_y = float (self.value ("close_inset_y"))
    bx = inset_x
    by = self.tile_y - inset_y
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("close", x, y, h_centre = -1, v_centre = 1)



#! /usr/bin/env python

from __future__ import absolute_import
import itertools, logtool, numbers, re, sys
from psfile import PSFile
from types import ListType
from .cmdio import CmdIO

class Sheet (CmdIO):

  @logtool.log_call (log_args = False)
  def __init__ (self, cfgs, paper, outline, sheet, page, fname):
    super (Sheet, self).__init__ (cfgs)
    self.re_var = re.compile (r"\$\{([A-Za-z0-9._]+)\}")
    self.paper = paper
    self.outline = outline
    self.cfgs = cfgs
    self.fname = fname
    self.fd = None
    # Type of sheet
    self.sheet = sheet
    self.page = page
    self.rotate = self.value ("rotate")
    # Size of grid
    self.num_x = int (self.value ("num_across_x"))
    self.num_y = int (self.value ("num_across_y"))
    # Size of tile
    self.tile_x = int (self.value ("tile_x"))
    self.tile_y = int (self.value ("tile_y"))
    # Size of rubber frame
    self.rubber_x = int (self.value ("rubber_x"))
    self.rubber_y = int (self.value ("rubber_y"))
    # Offsets for inner tile block
    self.x_off = (self.rubber_x - (self.tile_x * self.num_x)) / 2
    self.y_off = (self.rubber_y - (self.tile_y * self.num_y)) / 2

  @logtool.log_call
  def __enter__ (self):
    return self

  @logtool.log_call
  def __exit__ (self, t, v, tr):
    pass

  @logtool.log_call
  def open (self):
    self.fd = PSFile (self.fname, paper = self.paper, margin = 36)
    if self.rotate == "1":
      self.fd.append ("270 rotate")
      self.fd.append ("-%d %d translate" % (self.fd.height, 0))

  @logtool.log_call
  def close (self):
    if self.fd:
     self.fd.close ()
     self.fd = None

  @logtool.log_call (log_args = False)
  def _value_lookup (self, cfg, sl, k):
    base = cfg
    al = list ()
    # Descending list of sections
    for s in sl:
      a = base.get (s)
      if not a:
        break
      al.insert (0, a)
      base = a
    al.append (cfg.get ("DEFAULT"))
    for d in al:
      # print "K: ", k , "D: ", d, "V: [%s]" % d.get (k)
      if d.has_key (k):
        # print "Found %s / %s in %s" % (k, sl, cfg.my_name)
        return d.get (k)
    # print "Failed to find %s / %s in %s" % (k, sl, cfg.my_name)
    raise ValueError

  @logtool.log_call
  def _get_value (self, sl, k):
    for cfg in itertools.chain (self.cfgs):
      try:
        return self._value_lookup (cfg, sl, k)
      except: # pylint: disable=bare-except
        pass
    raise ValueError

  @logtool.log_call
  def value (self, n, x = 0, y = 0):
    sl = [self.sheet, self.page, "tile_%d.%d" % (x + 1, y + 1)]
    try:
      v = self._get_value (sl, n)
    except: # pylint: disable=bare-except
      print >> sys.stderr, (
        "FAULT: No definition found for: %s.%s.%s (%d, %d)"
        % (self.sheet, self.page, n, x + 1, y + 1))
      sys.exit (3)
    if v == "randomcolour": # Development ease
      from random import uniform
      v = (uniform (0, 1), uniform (0, 1), uniform (0, 1))
    elif isinstance (v, ListType): # So it satisfies the % operator
      v = tuple (v)
    else: # Substitute ${VAR} value
      while True:
        m = self.re_var.search (v)
        if not m:
          break
        k = m.group (1) # Variable name
        r = self.value (k, x, y) # Replacement token
        if isinstance (r, basestring):
          v = "%s%s%s" % (v[:m.start(0)], r, v[m.end(0):])
        else: # Scummy, but kinda handles the tuples
          v = r
          break
    return v

  @logtool.log_call
  def page_details (self):
    pass # Specialised in children

  @logtool.log_call
  def tile_details (self, x, y):
    pass # Specialised in children

  @logtool.log_call
  def tile_block (self):
    ox = self.x_off
    oy = self.y_off
    for x in xrange (self.num_x):
      for y in xrange (self.num_y):
        bx = (x * self.tile_x) + ox
        by = (y * self.tile_y) + oy
        self.push_tile (x, y, bx, by)
        self.tile_details (x, y)
        self.pop_tile ()

  @logtool.log_call
  def page_align (self):
    align_length = float (self.value ("align_length"))
    self.fd.append ("gsave")
    for i in [((self.rubber_x / 2.0, 0), (0, 0 - align_length)),
              ((self.rubber_x /2.0, self.rubber_y), (0, align_length)),
              ((0, self.rubber_y / 2.0), (0 - align_length, 0)),
              ((self.rubber_x, self.rubber_y / 2.0), (align_length, 0)),]:
      self.line ("align", 0, 0, i[0][0], i[0][1], i[1][0], i[1][1])
    self.fd.append ("grestore")

  @logtool.log_call
  def page_frame (self):
    self.box ("frame", 0, 0, 0, 0, self.rubber_x, self.rubber_y)

  @logtool.log_call
  def copyright (self):
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (0, self.rubber_y + 6))
    self.text ("copyright", 0, 0, v_centre = -1)
    self.fd.append ("grestore")
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (0, -12))
    self.text ("source_filename", 0, -12, v_centre = -1)
    self.fd.append ("grestore")
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (0, -24))
    self.text ("print_instruction", 0, -24, v_centre = -1)
    self.fd.append ("grestore")

  @logtool.log_call
  def push_tile (self, x, y, bx, by):
    self.fd.append ("gsave")
    self.fd.append ("%d %d translate" % (bx, by))
    if self.value ("outline", x, y) == "outline":
      self.box ("tile", x, y, 0, 0, self.tile_x, self.tile_y)
    self.fd.append ("newpath")

  @logtool.log_call
  def pop_tile (self):
    self.fd.append ("grestore")

  @logtool.log_call
  def line (self, typ, x, y, bx, by, w, h,
            dash_length = None, dash_space = 5, dash_start = 0):
    stroke_width = float (self.value ("%s_stroke" % typ, x, y))
    colour = self.value ("%s_colour" % typ, x, y)
    if colour != "transparent":
      self.fd.append ("gsave")
      if isinstance (dash_length, numbers.Number):
        self.fd.append ("1 setlinecap")
        self.fd.append ("[%s %s] %s setdash" % (dash_length, dash_space, dash_start))
      self.fd.append ("%s %s %s setrgbcolor" % colour)
      self.fd.append ("%f setlinewidth" % stroke_width)
      self.fd.append ("%f %f moveto" % (bx, by))
      self.fd.append ("%f %f lineto" % (bx + w, by + h))
      self.fd.append ("stroke")
      self.fd.append ("grestore")

  @logtool.log_call
  def box (self, typ, x, y, bx, by, w, h):
    stroke = float (self.value ("%s_stroke" % typ, x, y))
    stroke_colour = self.value ("%s_stroke_colour" % typ, x, y)
    colour_bg = self.value ("%s_colour" % typ, x, y)
    self.fd.append ("gsave")
    if colour_bg != "transparent":
      self.fd.append ("%s %s %s setrgbcolor" % colour_bg)
      self.fd.append ("%f %f %f %f rectfill" % (bx, by, w, h))
    if stroke_colour != "transparent":
      self.fd.append ("%s %s %s setrgbcolor" % stroke_colour)
      self.fd.append ("%f setlinewidth" % stroke)
      self.fd.append ("%f %f %f %f rectstroke" % (bx, by, w, h))
    self.fd.append ("grestore")

  @logtool.log_call
  def text (self, typ, x, y, h_centre = -1, v_centre = 1):
    line_height = float (self.value ("%s_line_height" % typ))
    self.fd.append ("gsave")
    self.fd.append ("currentpoint translate")
    self.fd.append ("/%s %s selectfont" % self.value ("%s_font" % typ, x, y))
    self.fd.append ("%s %s %s setrgbcolor"
                    % self.value ("%s_colour" % typ, x, y))
    tl = self.value (typ, x, y)
    if isinstance (tl, basestring):
      tl = tl.split ("\n") # pylint: disable=no-member
    if v_centre == 1:
      by = 0
    if v_centre == 0:
      by = line_height * ((float (len (tl) - 1) / 2))
      self.fd.append ("0 %d moveto" % by)
    if v_centre == -1:
      by = line_height * (len (tl) - 1)
      self.fd.append ("0 %d moveto" % by)
    tformat = {
      -1: "(%s) show",
      0: "(%s) dup stringwidth pop 2 div neg 0 rmoveto show",
      1: "(%s) dup stringwidth pop neg 0 rmoveto show",
      }
    for t in tl:
      self.fd.append (tformat[h_centre] % t)
      by -= line_height
      self.fd.append ("0 %d moveto" % by)
    self.fd.append ("grestore")

  @logtool.log_call
  def company_token_circle (self, x, y):
    token_radius = float (self.value ("token_radius", x, y))
    colour = self.value ("token_colour", x, y)
    stroke = float (self.value ("token_stroke", x, y))
    stroke_colour = self.value ("token_stroke_colour", x, y)
    ps = """
      gsave
      currentpoint translate
      newpath
      0 0 {token_radius} 0 360 arc
      gsave
      {colour} setrgbcolor
      fill
      grestore
      {stroke_colour} setrgbcolor
      {stroke} setlinewidth
      stroke
      grestore
    """.format (**{
      "token_radius": token_radius,
      "colour": "%s %s %s" % colour,
      "stroke_colour": "%s %s %s" % stroke_colour,
      "stroke": stroke,
    })
    self.fd.append (ps)

  @logtool.log_call
  def company_token (self, x, y):
    stroke = float (self.value ("token_stroke", x, y))
    stroke_colour = self.value ("token_stroke_colour", x, y)
    radius = float (self.value ("token_radius", x, y))
    stripe_angle = float (self.value ("token_stripe_angle", x, y))
    top_colour = self.value ("token_top_colour", x, y)
    top_stripe_colour = self.value ("token_top_stripe_colour", x, y)
    bottom_colour = self.value ("token_bottom_colour", x, y)
    bottom_stripe_colour = self.value ("token_bottom_stripe_colour", x, y)
    stripe_text_fudge = float (self.value ("token_stripe_text_fudge", x, y))
    self.company_token_circle (x, y)
    # Setup
    self.fd.append ("gsave")
    self.fd.append ("currentpoint translate")
    self.fd.append ("newpath")
    # Token top
    self.fd.append ("gsave")
    self.fd.append ("0 0 %f %f %f arc"
                    % (radius, 0 + stripe_angle, 180 - stripe_angle))
    self.fd.append ("closepath")
    self.fd.append ("gsave")
    self.fd.append ("%s %s %s setrgbcolor" % top_colour)
    self.fd.append ("fill")
    self.fd.append ("grestore")
    self.fd.append ("%s %s %s setrgbcolor" % stroke_colour)
    self.fd.append ("%f setlinewidth" % stroke)
    self.fd.append ("stroke")
    self.fd.append ("grestore")
    # Token top stripe
    if top_stripe_colour != "transparent":
      angle1 = float (self.value ("token_top_stripe_angle1", x, y))
      angle2 = float (self.value ("token_top_stripe_angle2", x, y))
      self.fd.append ("gsave")
      self.fd.append ("0 0 %f %f %f arc"
                      % (radius, 0 + angle1, 0 + angle2))
      self.fd.append ("0 0 %f %f %f arc"
                      % (radius, 180 - angle2, 180 - angle1))
      self.fd.append ("closepath")
      self.fd.append ("gsave")
      self.fd.append ("%s %s %s setrgbcolor" % top_stripe_colour)
      self.fd.append ("fill")
      self.fd.append ("grestore")
      self.fd.append ("%s %s %s setrgbcolor" % stroke_colour)
      self.fd.append ("%f setlinewidth" % stroke)
      self.fd.append ("stroke")
      self.fd.append ("grestore")
    # Token bottom
    self.fd.append ("gsave")
    self.fd.append ("0 0 %f %f %f arc"
                    % (radius, 180 + stripe_angle, 0 - stripe_angle))
    self.fd.append ("closepath")
    self.fd.append ("gsave")
    self.fd.append ("%s %s %s setrgbcolor" % bottom_colour)
    self.fd.append ("fill")
    self.fd.append ("grestore")
    self.fd.append ("%s %s %s setrgbcolor" % stroke_colour)
    self.fd.append ("%f setlinewidth" % stroke)
    self.fd.append ("stroke")
    self.fd.append ("grestore")
    # Token bottom stripe
    if bottom_stripe_colour != "transparent":
      angle1 = float (self.value ("token_bottom_stripe_angle1", x, y))
      angle2 = float (self.value ("token_bottom_stripe_angle2", x, y))
      self.fd.append ("gsave")
      self.fd.append ("0 0 %f %f %f arc"
                      % (radius, 180 + angle1, 180 + angle2))
      self.fd.append ("0 0 %f %f %f arc"
                      % (radius, 0 - angle2, 0 - angle1))
      self.fd.append ("closepath")
      self.fd.append ("gsave")
      self.fd.append ("%s %s %s setrgbcolor" % bottom_stripe_colour)
      self.fd.append ("fill")
      self.fd.append ("grestore")
      self.fd.append ("%s %s %s setrgbcolor" % stroke_colour)
      self.fd.append ("%f setlinewidth" % stroke)
      self.fd.append ("stroke")
      self.fd.append ("grestore")
    # Token text
    self.fd.append ("gsave")
    self.fd.append ("0 %f moveto" % (0 - stripe_text_fudge))
    self.text ("token_name", x, y, h_centre = 0, v_centre = 0)
    self.fd.append ("grestore")
    # Done
    self.fd.append ("grestore")

  def make (self):
    self.open ()
    self.page_align ()
    self.page_frame ()
    self.page_details ()
    self.tile_block ()
    self.copyright ()
    self.close ()



#!/usr/bin/env python

import clip, logtool, termcolor
from addict import Dict

COLOUR_ERROR = "red"
COLOUR_DEBUG = "blue"
COLOUR_INFO = "white"
COLOUR_INFO_BAD = "magenta"
COLOUR_WARN = "yellow"
COLOUR_REPORT = "green"
COLOUR_FIELDNAME = "cyan"
COLOUR_VALUE = "green"

class CmdIO (object):

  @logtool.log_call
  def __init__ (self, *args, **kwargs): # pylint: disable = W0613
    self.conf = Dict ()

  @logtool.log_call
  def colourise (self, msg, colour):
    return (termcolor.colored (msg, colour) if not self.conf.nocolour
            else msg)

  @logtool.log_call
  def debug (self, msg):
    if not self.conf.quiet and self.conf.verbose:
      clip.echo (self.colourise (msg, COLOUR_DEBUG))

  @logtool.log_call
  def info (self, msg, err = False):
    if not self.conf.quiet:
      clip.echo (self.colourise (msg,
                                 COLOUR_INFO if not err else COLOUR_INFO_BAD))

  @logtool.log_call
  def error (self, msg):
    if not self.conf.quiet:
      clip.echo (self.colourise (msg, COLOUR_ERROR))

  @logtool.log_call
  def warn (self, msg):
    if not self.conf.quiet:
      clip.echo (self.colourise ("WARNING: " + msg, COLOUR_WARN))

  @logtool.log_call
  def report (self, msg, err = False):
    if not self.conf.quiet:
      clip.echo (self.colourise (msg, COLOUR_REPORT if not err
                                 else COLOUR_ERROR))

  @logtool.log_call
  def report_field (self, field, value, err = False):
    if not self.conf.quiet:
      clip.echo ("%s: %s" % (
        self.colourise (field, COLOUR_FIELDNAME),
        self.colourise (value,
                        COLOUR_VALUE if not err else COLOUR_ERROR),))



#! /usr/bin/env python

from ._version import get_versions
__version__ = get_versions()['version']
__version_info__ = get_versions ()
del get_versions

from xxpaper.sheet import Sheet
from xxpaper.charter import Charter
from xxpaper.private import Private
from xxpaper.share import Share
from xxpaper.token import Token
from xxpaper.train import Train
from xxpaper.market import Market
from xxpaper.market15 import Market15



#! /usr/bin/env python

import logtool, math
from xxpaper.sheet import Sheet

class Token (Sheet):

  @logtool.log_call
  def tile_details (self, x, y):
    self.top (x, y)
    self.top_stripe (x, y)
    self.bottom (x, y)
    self.bottom_stripe (x, y)
    self.text_stripe (x, y)
    self.token_circles (x, y)

  @logtool.log_call
  def bottom (self, x, y):
    bx = self.tile_x
    by = self.tile_y / 2
    self.fd.append ("gsave")
    self.fd.append ("%s %s %s setrgbcolor"
                    % self.value ("token_bottom_colour",x ,y))
    self.fd.append ("%d %d %d %d rectfill" % (0, 0, bx, by))
    self.fd.append ("grestore")

  @logtool.log_call
  def bottom_stripe (self, x, y):
    stripe_colour = self.value ("token_bottom_stripe_colour", x, y)
    if stripe_colour == "transparent":
      return
    bx = self.tile_x
    # by = self.tile_y / 2
    radius = float (self.value ("token_radius", x, y))
    angle1 = float (self.value ("token_bottom_stripe_angle1", x, y))
    angle2 = float (self.value ("token_bottom_stripe_angle2", x, y))
    height1 = math.sin (math.radians (angle1)) * radius
    height2 = math.sin (math.radians (angle2)) * radius
    self.fd.append ("gsave")
    self.fd.append ("%s %s %s setrgbcolor" % stripe_colour)
    self.fd.append ("%f %f %f %f rectfill"
                    % (0, (self.tile_y / 2) - height2,
                       bx, height2 - height1))
    self.fd.append ("grestore")

  @logtool.log_call
  def top (self, x, y):
    bx = self.tile_x
    # by = float (self.tile_y / 2)
    self.fd.append ("gsave")
    self.fd.append ("%s %s %s setrgbcolor"
                    % self.value ("token_top_colour", x, y))
    self.fd.append ("%f %f %f %f rectfill"
                    % (0, self.tile_y / 2,
                       bx, self.tile_y / 2))
    self.fd.append ("grestore")

  @logtool.log_call
  def top_stripe (self, x, y):
    stripe_colour = self.value ("token_top_stripe_colour", x, y)
    if stripe_colour == "transparent":
      return
    bx = self.tile_x
    # by = float (self.tile_y / 2)
    radius = float (self.value ("token_radius", x, y))
    angle1 = float (self.value ("token_top_stripe_angle1", x, y))
    angle2 = float (self.value ("token_top_stripe_angle2", x, y))
    height1 = math.sin (math.radians (angle1)) * radius
    height2 = math.sin (math.radians (angle2)) * radius
    self.fd.append ("gsave")
    self.fd.append ("%s %s %s setrgbcolor" % stripe_colour)
    self.fd.append ("%f %f %f %f rectfill"
                    % (0, (self.tile_y / 2) + height1,
                       bx, height2 - height1))
    self.fd.append ("grestore")

  @logtool.log_call
  def text_stripe (self, x, y):
    stroke = float (self.value ("token_stroke"))
    stroke_colour = self.value ("token_stroke_colour")
    stripe_height = float (self.value ("text_stripe_height"))
    bx = self.tile_x
    by = stripe_height
    self.fd.append ("gsave")
    self.fd.append ("%s %s %s setrgbcolor"
                    % self.value ("token_colour", x, y))
    self.fd.append ("%f %f %f %f rectfill"
                    % (0, float (self.tile_y / 2) - (stripe_height / 2),
                       bx, by))
    self.fd.append ("%s %s %s setrgbcolor" % stroke_colour)
    self.fd.append ("%f setlinewidth" % stroke)
    self.fd.append ("%f %f %f %f rectstroke"
                    % (0, float (self.tile_y / 2) - (stripe_height / 2),
                       bx, by))
    self.fd.append ("grestore")

  @logtool.log_call
  def token_circles (self, x, y):
    space_x = float (self.value ("token_space_x"))
    radius = float (self.value ("token_radius"))
    ox = space_x
    oy = float (self.tile_y / 2)
    spacing = space_x + (radius * 2)
    count = int (self.value ("token_count", x, y))
    self.fd.append ("gsave")
    for i in xrange (1, count + 1):
      bx = ox + (i * spacing)
      by = oy
      self.fd.append ("%f %f moveto" % (bx, by))
      self.company_token (x, y)
    self.fd.append ("grestore")



#! /usr/bin/env python

import logtool
from xxpaper.sheet import Sheet

class Share (Sheet):

  @logtool.log_call (log_args = False)
  def __init__ (self, cfgs, paper, outline, sheet, page, fname):
    super (Share, self).__init__ (cfgs, paper, outline, sheet, page, fname)
    # Offsets within tile
    self.side_stripe_inset_x = float (self.value ("side_stripe_inset_x"))
    self.side_stripe_width = float (self.value ("side_stripe_width"))
    self.type_stripe_height = float (self.value ("type_stripe_height"))
    self.type_stripe_inset_x = float (self.value ("type_stripe_inset_x"))
    self.type_stripe_inset_y = float (self.value ("type_stripe_inset_y"))
    self.type_stripe_inset_y_fudge = float (
      self.value ("type_stripe_inset_y_fudge"))
    self.type_stripe_width = (self.x_off + self.tile_x -
                              self.type_stripe_inset_x)
    self.side_split_count = int (self.value ("side_split_count"))

  @logtool.log_call
  def page_details (self):
    self.side_stripe ()
    self.side_stripe_split ()
    self.type_stripe ()

  @logtool.log_call
  def tile_details (self, x, y):
    self.type_size (x, y)
    self.type_note (x, y)
    self.type_desc (x, y)
    self.tokens (x, y)
    self.title (x, y)
    self.description (x, y)

  @logtool.log_call
  def side_stripe (self):
    ox = self.x_off + self.side_stripe_inset_x
    for x in xrange (self.num_x):
      bx = (self.tile_x * x) + ox
      self.box ("stripe", 0, 0, bx, 0, self.side_stripe_width, self.rubber_y)

  @logtool.log_call
  def side_stripe_split (self):
    dash_length = self.value ("side_split_dash_length")
    dash_space = self.value ("side_split_dash_space")
    place = self.value ("side_split_place")
    gap = float (self.value ("side_split_gap"))

    if place == "left":
      ox = self.x_off + self.side_stripe_inset_x
    elif place == "centre":
      gap = self.side_stripe_width / (self.side_split_count + 1)
      ox = self.x_off + self.side_stripe_inset_x
    elif place == "right":
      ox = (self.x_off + self.side_stripe_inset_x + self.type_stripe_width
            - (gap * self.side_split_count))

    for x in xrange (self.num_x):
      for i in xrange (self.side_split_count):
        bx = (self.tile_x * x) + ox + ((i + 1) * gap)
        self.line ("side_split", 0, 0, bx, 0,
                   0, self.rubber_y,
                   dash_length = 10, dash_space = 5, dash_start = 5 * (i % 2))

  @logtool.log_call
  def type_stripe (self):
    oy = self.y_off + self.type_stripe_inset_y
    for y in xrange (self.num_y):
      by = (y * self.tile_y) + oy
      self.box ("type", 0, y, 0, by,
                self.type_stripe_width, self.type_stripe_height)

  @logtool.log_call
  def type_size (self, x, y):
    type_colour = self.value ("type_colour", x, y)
    if type_colour == "transparent":
      bx = self.side_stripe_inset_x
      by = self.type_stripe_inset_y
      self.box ("size_box", x, y, bx, by,
                self.side_stripe_width, self.type_stripe_height)
    bx = self.side_stripe_inset_x + (self.side_stripe_width / 2)
    by = self.type_stripe_inset_y + self.type_stripe_inset_y_fudge
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("size", x, y, h_centre = 0, v_centre = 1)

  @logtool.log_call
  def type_note (self, x, y):
    inset_x = float (self.value ("type_note_inset_x"))
    bx = (self.side_stripe_inset_x + self.side_stripe_width + inset_x)
    by = self.type_stripe_inset_y + self.type_stripe_inset_y_fudge
    if self.value ("note", x, y):
      self.fd.append ("%f %f moveto" % (bx, by))
      self.text ("note", x, y, h_centre = -1, v_centre = 1)

  @logtool.log_call
  def type_desc (self, x, y):
    inset_x = float (self.value ("type_desc_inset_x"))
    bx = self.type_stripe_width - inset_x - self.x_off
    by = self.type_stripe_inset_y + self.type_stripe_inset_y_fudge
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("desc", x, y, h_centre = 1, v_centre = 1)

  @logtool.log_call
  def tokens (self, x, y):
    inset_y = float (self.value ("token_inset_y"))
    ox = self.side_stripe_inset_x + (self.side_stripe_width / 2)
    oy = (self.type_stripe_inset_y + self.type_stripe_height + inset_y)
    gap = self.tile_y - oy - inset_y
    count = int (self.value ("token_count", x, y))
    spacing = gap / (count + 1)
    for i in xrange (1, count + 1):
      bx = ox
      by = oy + (i * spacing)
      self.fd.append ("%f %f moveto" % (bx, by))
      self.company_token (x, y)

  @logtool.log_call
  def title (self, x, y):
    inset_x = float (self.value ("title_inset_x"))
    inset_y = float (self.value ("title_inset_y"))
    o = (self.side_stripe_inset_x + self.side_stripe_width + inset_x)
    w = self.tile_x - o
    bx = (o + (w / 2))
    by = self.tile_y - inset_y
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("title", x, y, h_centre = 0, v_centre = 1)

  @logtool.log_call
  def description (self, x, y):
    inset_x = float (self.value ("description_inset_x"))
    inset_y = float (self.value ("description_inset_y"))
    o = (self.side_stripe_inset_x + self.side_stripe_width + inset_x)
    w = self.tile_x - o
    bx = (o + (w / 2))
    by = inset_y
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("description", x, y, h_centre = 0, v_centre = -1)



#! /usr/bin/env python

import logtool
from xxpaper.sheet import Sheet

class Charter (Sheet):

  @logtool.log_call (log_args = False)
  def __init__ (self, cfgs, paper, outline, sheet, page, fname):
    super (Charter, self).__init__ (cfgs, paper, outline, sheet, page, fname)
    # Offsets within tile
    self.box_inset_x = float (self.value ("box_inset_x"))
    self.box_inset_y = float (self.value ("box_inset_y"))
    self.box_radius = float (self.value ("box_radius"))
    self.stripe_height = float (self.value ("stripe_height"))
    self.stripe_radius = float (self.value ("stripe_radius"))
    self.title_inset_x = float (self.value ("title_inset_x"))

  @logtool.log_call
  def page_details (self):
    self.macro_roundbox ()

  @logtool.log_call
  def tile_details (self, x, y):
    self.charter_box (x, y)
    self.charter_stripe (x, y)
    self.centre_stripe (x, y)
    self.charter_stripe_token (x, y)
    self.title (x, y)
    self.token_circles (x, y)
    self.desc1 (x, y)
    self.desc2 (x, y)
    self.note1 (x, y)
    self.note2 (x, y)

  @logtool.log_call
  def macro_roundbox (self):
    self.fd.define (
      "roundbox",
      """
%%Title: Roundbox
% Devised in 2000 by George Spowart
% http://compgroups.net/comp.lang.postscript/how-to-draw-rectangles-with-rounded-corn/1828790
%/roundbox {
/radius exch def
/height exch def
/width exch def
0 radius moveto
0 height width height radius arcto 4 {pop} repeat
width height width 0 radius arcto 4 {pop} repeat
width 0 0 0 radius arcto 4 {pop} repeat
0 0 0 height radius arcto 4 {pop} repeat
closepath
% } def
""")

  @logtool.log_call
  def charter_box (self, x, y):
    bx = self.box_inset_x
    by = self.box_inset_y
    self.fd.append ("%f %f moveto" % (bx, by))
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.fd.append ("currentpoint translate")
    self.fd.append ("%f %f %f roundbox"
                    % (self.tile_x - (self.box_inset_x * 2),
                       self.tile_y - (self.box_inset_y * 2),
                       self.box_radius))
    self.fd.append ("gsave")
    self.fd.append ("%s %s %s setrgbcolor" % self.value ("box_colour", x, y))
    self.fd.append ("fill")
    self.fd.append ("grestore")
    self.fd.append ("%s setlinewidth" % self.value ("box_stroke", x, y))
    self.fd.append ("%s %s %s setrgbcolor"
                    % self.value ("box_stroke_colour", x, y))
    self.fd.append ("stroke")
    self.fd.append ("grestore")

  @logtool.log_call
  def charter_stripe (self, x, y):
    bx = self.box_inset_x
    by = self.tile_y - self.box_inset_y - self.stripe_height
    self.fd.append ("%f %f moveto" % (bx, by))
    self.fd.append ("gsave")
    self.fd.append ("currentpoint translate")
    self.fd.append ("%f %f %f roundbox"
                    % (self.tile_x - (self.box_inset_x * 2),
                       self.stripe_height,
                       self.stripe_radius))
    self.fd.append ("gsave")
    self.fd.append ("%s %s %s setrgbcolor"
                    % self.value ("stripe_colour", x, y))
    self.fd.append ("fill")
    self.fd.append ("grestore")
    self.fd.append ("%s setlinewidth" % self.value ("stripe_stroke", x, y))
    self.fd.append ("%s %s %s setrgbcolor"
                    % self.value ("stripe_stroke_colour", x, y))
    self.fd.append ("closepath")
    self.fd.append ("stroke")
    self.fd.append ("grestore")

  @logtool.log_call
  def charter_stripe_token (self, x, y):
    bx = self.box_inset_x + self.stripe_radius
    by = self.tile_y - self.box_inset_y - self.stripe_radius
    self.fd.append ("%f %f moveto" % (bx, by))
    self.company_token (x, y)

  @logtool.log_call
  def title (self, x, y):
    title_inset_y_fudge = float (self.value ("title_inset_y_fudge", x, y))
    bx = (self.box_inset_x + ((self.tile_x - (self.box_inset_x * 2)) / 2)
          + self.title_inset_x)
    by = (self.tile_y - self.box_inset_x - self.box_radius
          - title_inset_y_fudge)
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("title", x, y, h_centre = 0, v_centre = 0)
    self.fd.append ("grestore")

  @logtool.log_call
  def token_circles (self, x, y):
    token_inset_x = float (self.value ("token_inset_x", x, y))
    token_inset_y = float (self.value ("token_inset_y", x, y))
    token_radius = float (self.value ("token_radius", x, y))
    ox = (self.box_inset_x
          + token_inset_x + token_radius)
    oy = (self.tile_y - self.box_inset_y - self.stripe_height
          - token_inset_y)
    gap = oy - token_inset_y
    count = int (self.value ("token_count", x, y))
    token_spacing = int (self.value ("charter_token_spacing", x))
    spacing = gap / token_spacing if token_spacing else gap / (count + 1)
    self.fd.append ("gsave")
    for i in xrange (1, count + 1):
      bx = ox
      by = oy - (i * spacing)
      self.fd.append ("%f %f moveto" % (bx, by))
      self.company_token_circle (x, y)
      price_height_fudge = float (self.value ("price%s_height_fudge" % i, x, y))
      self.fd.append ("%f %f moveto" % (bx, by - price_height_fudge))
      self.text ("price%s" % i, x, y, h_centre = 0, v_centre = 1)
    self.fd.append ("grestore")

  @logtool.log_call
  def centre_stripe (self, x, y):
    bx = self.tile_x / 2
    by = self.box_inset_y
    h = self.tile_y - by - self.box_inset_y - self.stripe_height
    self.line ("centre", x, y, bx, by, 0, h)

  @logtool.log_call
  def desc1 (self, x, y):
    centre = self.value ("centre_colour", x, y)
    if centre == "transparent":
      bx = self.tile_x / 2
    else:
      bx = self.tile_x / 4
    by = (self.tile_y - self.box_inset_y - self.box_radius
          - self.stripe_height)
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("desc1", x, y, h_centre = 0, v_centre = 0)
    self.fd.append ("grestore")

  @logtool.log_call
  def desc2 (self, x, y):
    centre = self.value ("centre_colour", x, y)
    if centre == "transparent":
      return
    bx = (self.tile_x * 3) / 4
    by = (self.tile_y - self.box_inset_y - self.box_radius
          - self.stripe_height)
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("desc2", x, y, h_centre = 0, v_centre = 0)
    self.fd.append ("grestore")

  @logtool.log_call
  def note1 (self, x, y):
    centre = self.value ("centre_colour", x, y)
    if centre == "transparent":
      bx = self.tile_x / 2
    else:
      bx = self.tile_x / 4
    by = self.box_inset_y + (self.box_radius / 2)
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("note1", x, y, h_centre = 0, v_centre = -1)
    self.fd.append ("grestore")

  @logtool.log_call
  def note2 (self, x, y):
    centre = self.value ("centre_colour", x, y)
    if centre == "transparent":
      return # No second side
    bx = (self.tile_x * 3) / 4
    by = self.box_inset_y + (self.box_radius / 2)
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("note2", x, y, h_centre = 0, v_centre = -1)
    self.fd.append ("grestore")



#! /usr/bin/env python

import logtool
from xxpaper.sheet import Sheet

class Market15 (Sheet):

  @logtool.log_call
  def page_details (self):
    pass

  @logtool.log_call
  def tile_details (self, x, y):
    self.type_desc (x, y)

  @logtool.log_call
  def type_desc (self, x, y):
    bx = self.tile_x / 2
    by = self.tile_y / 10
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("desc", x, y, h_centre = 0, v_centre = -1)
    self.fd.append ("grestore")
    bx = self.tile_x / 2
    by = 9 * self.tile_y / 10
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.fd.append ("180 rotate")
    self.text ("desc", x, y, h_centre = 0, v_centre = -1)
    self.fd.append ("grestore")

  @logtool.log_call
  def tile_block (self):
    ox = self.x_off
    oy = self.y_off
    half_ox = int (self.tile_x / 2)
    for x in xrange (self.num_x):
      for y in xrange (self.num_y):
        bx = (x * self.tile_x) + ox + ((y % 2) * half_ox)
        by = (y * self.tile_y) + oy
        self.push_tile (x, y, bx, by)
        self.tile_details (x, y)
        self.pop_tile ()



#! /usr/bin/env python

from setuptools import setup, find_packages
import versioneer

setup (
    name = "xxpaper",
    version = versioneer.get_version (),
    description = "Tools for generating files of 18xx papers.",
    long_description = "Tools for generating printable files of shares, privates, trains etc for 18xx and like games.",
    cmdclass = versioneer.get_cmdclass (),
    classifiers = [],
    keywords = "18xx, game files",
    author = "J C Lawrence",
    author_email = "claw@kanga.nu",
    url = "https://github.com/clearclaw/xxpaper",
    license = "Creative Commons Attribution-ShareAlike 3.0 Unported",
    packages = find_packages (exclude = ["tests",]),
    include_package_data = True,
    zip_safe = False,
    install_requires = [line.strip ()
                        for line in file ("requirements.txt").readlines ()
                    ] + ["psfile",],
  entry_points = {
    "console_scripts": [
      "xxpaper = xxpaper.main:main",
      ],
    },
  )



#! /usr/bin/env python

import logtool
from xxpaper.sheet import Sheet

class Market (Sheet):

  @logtool.log_call
  def page_details (self):
    pass

  @logtool.log_call
  def tile_details (self, x, y):
    self.type_desc (x, y)

  @logtool.log_call
  def type_desc (self, x, y):
    bx = self.tile_x / 2
    by = self.tile_y / 10
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.text ("desc", x, y, h_centre = 0, v_centre = -1)
    self.fd.append ("grestore")
    bx = self.tile_x / 2
    by = 9 * self.tile_y / 10
    self.fd.append ("gsave")
    self.fd.append ("%f %f moveto" % (bx, by))
    self.fd.append ("180 rotate")
    self.text ("desc", x, y, h_centre = 0, v_centre = -1)
    self.fd.append ("grestore")
